//Module to deserialize serial data
//DJS 6_20_16

`timescale 1ps / 1ps
module deserializer
(
	rx_start_p,
	rx_start_n,
	clk_msb,
	clk_sdr,
	clk_ddr,
	sdo,
	parallel_data,
	data_latch,
	ddr,
	num_bits
);

input rx_start_p,rx_start_n,clk_msb,clk_sdr,clk_ddr,sdo,data_latch,ddr,num_bits;
output parallel_data;

reg [15:0] parallel_data,bits;
reg [7:0] bits_odd,bits_even;
reg [5:0] cur_state,next_state,cur_state_p,next_state_p,cur_state_n,next_state_n;

wire [5:0] num_bits;
wire rx_start_p,rx_start_n;

parameter bit_0 = 5'd0;
parameter bit_1 = 5'd1;
parameter bit_2 = 5'd2;
parameter bit_3 = 5'd3;
parameter bit_4 = 5'd4;
parameter bit_5 = 5'd5;
parameter bit_6 = 5'd6;
parameter bit_7 = 5'd7;
parameter bit_8 = 5'd8;
parameter bit_9 = 5'd9;
parameter bit_10 = 5'd10;
parameter bit_11 = 5'd11;
parameter bit_12 = 5'd12;
parameter bit_13 = 5'd13;
parameter bit_14 = 5'd14;

initial
begin
  next_state = bit_14;
  next_state_p = bit_7;
  next_state_n = bit_6;
end
	
always @(posedge clk_msb)
begin
	bits[15] <= sdo;
	bits_odd[7] <= sdo;
end	
	
always @(negedge clk_sdr) //SDR mode; load the newest bit into the serial bit array on each falling edge of 'clk'
begin
	cur_state = next_state;		 		
	case (cur_state)
	
	bit_14:  //MSB first
	begin	
		if(rx_start_n)
      begin			
		  bits[14] <= sdo;
	     next_state <= bit_13;
      end	
    else
      next_state <= bit_14;
   end
    	
	bit_13:
   begin						
		bits[13] <= sdo;		
	 next_state <= bit_12;
   end	
  	
	bit_12:
   begin						
		bits[12] <= sdo;			
	 next_state <= bit_11;
   end	
  	
	bit_11:
   begin						
		bits[11] <= sdo;		
	 next_state <= bit_10;
   end	
  	
	bit_10:
   begin						
		bits[10] <= sdo;		
	 next_state <= bit_9;
   end	
  	
	bit_9:
   begin						
		bits[9] <= sdo;
	 next_state <= bit_8;
   end	
  	
	bit_8:
   begin						
		bits[8] <= sdo;		
	 next_state <= bit_7;
   end	
  	
	bit_7:
   begin						
		bits[7] <= sdo;		
	 next_state <= bit_6;
   end	
  	
	bit_6:
   begin						
		bits[6] <= sdo;	
	 next_state <= bit_5;
   end	
  	
	bit_5:
   begin						
		bits[5] <= sdo;		
	 next_state <= bit_4;
   end	
    	
	bit_4:
   begin						
		bits[4] <= sdo;
		next_state <= bit_3;
   end	
    	
	bit_3:
   begin					
		bits[3] <= sdo;
		next_state <= bit_2;
   end	
    	
	bit_2:
   begin		
		bits[2] <= sdo;		
		next_state = bit_1;
   end	
    	
	bit_1:
   begin						
		bits[1] <= sdo;	
		next_state <= bit_0;
   end	
    	
	bit_0:  //LSB last
   begin	
		bits[0] <= sdo;
		next_state <= bit_14;
	end	
	    
default 	: next_state <= bit_14;

endcase
end
 
always @(posedge clk_ddr) //DDR mode; load the newest even bit into the serial bit array on each rising edge of 'clk'
begin
	cur_state_p = next_state_p;		 		
	case (cur_state_p)
	
	bit_7:  //MSB first
	begin	
		if(rx_start_p)
      begin			
		  bits_even[7] <= sdo;
	     next_state_p <= bit_6;
      end	
    else
      next_state_p <= bit_7;
   end
		
	bit_6:
   begin	
		bits_even[6] <= sdo;
		next_state_p <= bit_5;
   end	
    	
	bit_5:
   begin						
		bits_even[5] <= sdo;		
		next_state_p <= bit_4;
   end	
  	
	bit_4:
   begin						
		bits_even[4] <= sdo;			
		next_state_p <= bit_3;
   end	
  	
	bit_3:
   begin						
		bits_even[3] <= sdo;		
		next_state_p <= bit_2;
   end	
  	
	bit_2:
   begin						
		bits_even[2] <= sdo;		
		next_state_p <= bit_1;
   end	
 
	bit_1:
   begin						
		bits_even[1] <= sdo;	
		next_state_p <= bit_0;
   end	
    	
	bit_0:  //LSB last
   begin	
		bits_even[0] <= sdo;
		next_state_p <= bit_7;
	end	
	    
default 	: next_state_p <= bit_7;

endcase
end

always @(negedge clk_ddr) //DDR mode; load the newest odd bit into the serial bit array on each falling edge of 'clk'
begin
	cur_state_n = next_state_n;		 		
	case (cur_state_n)
	
	bit_6:  //MSB first
	begin	
		if(rx_start_n)
      begin			
		  bits_odd[6] <= sdo;
	     next_state_n <= bit_5;
      end	
    else
      next_state_n <= bit_6;
   end
		
	bit_5:
   begin						
		bits_odd[5] <= sdo;		
		next_state_n <= bit_4;
   end	
  	
	bit_4:
   begin						
		bits_odd[4] <= sdo;			
		next_state_n <= bit_3;
   end	
  	
	bit_3:
   begin						
		bits_odd[3] <= sdo;		
		next_state_n <= bit_2;
   end	
  	
	bit_2:
   begin						
		bits_odd[2] <= sdo;		
		next_state_n <= bit_1;
   end	
 
	bit_1:
   begin						
		bits_odd[1] <= sdo;	
		next_state_n <= bit_0;
   end	
    	
	bit_0:  //LSB last
   begin	
		bits_odd[0] <= sdo;
		next_state_n <= bit_6;
	end	
	    
default 	: next_state_n <= bit_6;

endcase
end

always @ (posedge data_latch)
begin
  if(ddr)
  begin
    parallel_data[15] = bits_odd[7];
    parallel_data[14] = bits_even[7];    
    parallel_data[13] = bits_odd[6];
    parallel_data[12] = bits_even[6]; 
    parallel_data[11] = bits_odd[5];
    parallel_data[10] = bits_even[5];    
    parallel_data[9] = bits_odd[4];
    parallel_data[8] = bits_even[4]; 
    parallel_data[7] = bits_odd[3];
    parallel_data[6] = bits_even[3];    
    parallel_data[5] = bits_odd[2];
    parallel_data[4] = bits_even[2]; 
    parallel_data[3] = bits_odd[1];
    parallel_data[2] = bits_even[1];    
    parallel_data[1] = bits_odd[0];
    parallel_data[0] = bits_even[0]; 
  end
  
  else
  begin
    parallel_data[15] = bits[15];
    parallel_data[14] = bits[14];    
    parallel_data[13] = bits[13];
    parallel_data[12] = bits[12]; 
    parallel_data[11] = bits[11];
    parallel_data[10] = bits[10];    
    parallel_data[9] = bits[9];
    parallel_data[8] = bits[8]; 
    parallel_data[7] = bits[7];
    parallel_data[6] = bits[6];    
    parallel_data[5] = bits[5];
    parallel_data[4] = bits[4]; 
    parallel_data[3] = bits[3];
    parallel_data[2] = bits[2];    
    parallel_data[1] = bits[1];
    parallel_data[0] = bits[0]; 
	end
end

endmodule
